use sqlx::SqlitePool;

use crate::types::uuid::Uuid;

use super::events::Event;

pub struct Room {
    uuid: Uuid,
    name: String,
}

impl Room {
    fn new(name: &str) -> anyhow::Result<Self> {
        Ok(Self {
            uuid: uuid::Uuid::new_v4().into(),
            name: name.to_owned(),
        })
    }

    pub async fn create(&self, conn: &SqlitePool) -> anyhow::Result<Self> {
        Ok(sqlx::query_as!(
            Self,
            "insert into rooms(uuid, name)
            values(?, ?)
            returning uuid as 'uuid: Uuid', name",
            self.uuid,
            self.name
        )
        .fetch_one(conn)
        .await?)
    }

    pub async fn events(&self, conn: &SqlitePool) -> anyhow::Result<Vec<Event>> {
        Event::all_for_room(conn, self).await
    }

    /// Get a reference to the room's uuid.
    #[must_use]
    pub fn uuid(&self) -> &Uuid {
        &self.uuid
    }
}
